(define-module (ui net-domain ajax)
  :use-module (alterator ajax)
  :use-module (alterator woo)
  :use-module (alterator effect)
  :use-module (ice-9 regex)
  :use-module (ice-9 format)
  :export (on-load))

(define (clear-passwords)
  (form-update-value "ad_password" "")
  (form-update-value "ad_password2" "")
  (form-update-value "ipa_password" "")
  (form-update-value "ipa_password2" ""))

; Wait 10s after install
(define *after-install-wait* 15)

(define (update-ipa-services data)
  (for-each
   (lambda(name)
	 (form-update-value name
						(if (woo-get-option data (string->symbol name))
							(_ "UP")
							(_ "DOWN"))))
   '("ipa_named_status" "ipa_ldap_status" "ipa_krb_status"
	 "ipa_web_status" "ipa_cache_status" "ipa_pki_status"
	 "ipa_otp_status" "ipa_custodia_status"))
  (form-update-value "ipa_web_uri" (woo-get-option data 'ipa_web_uri)))

(define (update-ipa-progress-target data)
  (form-update-value "ipa_install_target"
					 (let ((target (woo-get-option data 'ipa_install_target)))
					   (cond
						((equal? target "install") (_ "Install"))
						((equal? target "uninstall") (_ "Uninstall"))
						(target)))))

(define (ui-read updating)
  (catch/message
   (lambda()
	 (let ((data (woo-read-first "/net-domain")))
	   (form-update-value-list data)
	   (update-ipa-services data)
	   (update-ipa-progress-target data)
	   (clear-passwords)
	   (form-update-visibility "altdomain_available"
							   (woo-get-option data 'altdomain_available))
	   (form-update-visibility '("altdomain_unavailable"
								 "altdomain_unavailable_reason")
							   (not (woo-get-option data 'altdomain_available)))
	   (form-update-visibility "ad_available"
							   (woo-get-option data 'ad_available))
	   (form-update-visibility '("ad_unavailable" "ad_unavailable_reason")
							   (not (woo-get-option data 'ad_available)))
	   (form-update-visibility "ipa_available"
							   (woo-get-option data 'ipa_available))
	   (form-update-visibility '("ipa_unavailable" "ipa_unavailable_reason")
							   (not (woo-get-option data 'ipa_available)))
	   (let update-ipa ((data data) (wait 0))
		 (form-update-visibility "ipa-status-options"
								 (and (not (woo-get-option data 'ipa_install_running))
									  (or (woo-get-option data 'ipa_installed)
										  (woo-get-option data 'ipa_available))))
		 (form-update-visibility "ipa-status-services"
								 (and (not (woo-get-option data 'ipa_install_running))
									  (woo-get-option data 'ipa_installed)))
		 (if (woo-get-option data 'ipa_install_running)
			 (begin
			   (form-update-value "domain_type" "ipa")
			   (form-update-visibility "ipa-status-progress" #t)
			   (form-update-value-list '("ipa_install_progress"
										 "ipa_install_message")
									   data)
			   (update-ipa-progress-target data)
			   (form-update-activity "ipa_install_stop" #t)
			   (form-set-timeout (lambda()
								   (catch/message
									(lambda()
									  (update-ipa (woo-read-first "/net-domain")
												  *after-install-wait*))))
								 1.0))
			 (begin
			   (update-ipa-services data)
			   (form-update-visibility "ipa-status-progress" #f)
			   (let ((ecode (woo-get-option data 'ipa_install_exitcode)))
				 (if (and ecode (not (or (string-null? ecode)
										 (equal? ecode "0")))
						  (or updating (> wait 0)))
					 (form-warning (_ (regexp-substitute
					 					  #f
					 					  (string-match
					 					   "ERROR:\\s*"
					 					   (woo-get-option data 'ipa_install_message))
					 					  'post)))
					 (begin
					   (if (> wait 0)
						   (form-set-timeout (lambda()
											   (catch/message
												(lambda()
												  (update-ipa
												   (woo-read-first "/net-domain")
												   (- wait 1)))))
											 1.0)))))))
		 (ui-set-type))))))

(define (restore-configs)
  (if (and (form-value "restore_conf")
       (form-confirm (_ "Are you sure you want to restore the default configuration file (krb5.conf)?")))
       (catch/message
			(lambda()
				(woo-write "/net-domain/restore_configs")
			)
       )
  )
  (ui-write)
)

(define (ui-write)
  (catch/message
   (lambda()
	 (let ((params (list "/net-domain"
	 					 'language (form-value "language")
	 					 'domain_type (form-value "domain_type")
	 					 'domain (form-value "domain")
	 					 'ad_dns (form-value "ad_dns"))))
	   ;; Check dot for ait least two parts in domain (ALT #40566)
	   (if (and (not (equal? (form-value "domain_type") "dns"))
		    (not (string-contains (form-value "domain") ".")))
	       (woo-error (_ "Domain name should have at least two labels separated by dot")))
	   (cond
	 	((equal? (form-value "domain_type") "ad")
	 	 (if (not (equal? (form-value "ad_password")
	 					  (form-value "ad_password2")))
	 		 (woo-error (_ "Passwords mismatch"))
	 		 (apply woo-write
	 				(append params
	 						(list 'ad_password (form-value "ad_password"))))))
	 	 ((equal? (form-value "domain_type") "ipa")
	 	  (if (not (equal? (form-value "ipa_password")
	 					   (form-value "ipa_password2")))
	 		  (woo-error (_ "Passwords mismatch"))
	 		  (apply woo-write
	 				 (append params
	 						 (list 'ipa_password (form-value "ipa_password"))))))
	 	 (#t (apply woo-write params))))
	 (ui-read #t))))

; show status pane on type change
(define (ui-set-type)
  (effect-show "altdomain-status" "domain_type" "altdomain")
  (effect-show "ad-status" "domain_type" "ad")
  (effect-show "ipa-status" "domain_type" "ipa")
  (update-effect))

(define (ipa-install-stop)
  (form-update-activity "ipa_install_stop" #f)
  (woo "stop_ipa" "/net-domain"))

(define (on-load)
  (ui-read #f)
  (init-effect)
  (form-update-activity "altdomain_unavailable" #f)
  (form-update-activity "ad_unavailable" #f)
  (form-update-activity "ipa_unavailable" #f)
  (form-bind "domain_type" "change" ui-set-type)
  (form-bind "submit" "click" restore-configs)
  (form-bind "reset" "click" (lambda() (ui-read #f)))
  (form-bind "ipa_install_stop" "click" ipa-install-stop))
